-- Cmo Paginar datos en SQL Server, 
-- tanto como ROW_NUMBER (acceso aleatorio) 
-- como con TOP (acceso secuencial)
--
--
-- GuilleSQL - Un portal sobre Microsoft SQL Server en Castellano
-- http://www.guillesql.es
--

-- *********************************************
-- ***  Crear Tabla para las Pruebas         ***
-- *********************************************
CREATE TABLE [dbo].[TBL_PAGINAS](
	[ID] [int] IDENTITY(1,1) NOT NULL,
	[TITULO] [varchar](50) NOT NULL,
	[FEC_PUBLICACION] [datetime2](7) NOT NULL,
    CONSTRAINT [PK_TBL_PAGINAS] PRIMARY KEY CLUSTERED ([ID] ASC)
) ON [PRIMARY]

GO


-- *********************************************
-- ***  Llenar la Tabla con datos de Prueba  ***
-- *********************************************
SET NOCOUNT ON;

DECLARE @CONT INT;
SET @CONT = 1

WHILE (@CONT<=10000)
BEGIN
   INSERT INTO dbo.TBL_PAGINAS (TITULO, FEC_PUBLICACION) VALUES (@CONT, CAST('19900101' AS DATETIME)+@CONT);
   SET @CONT = @CONT + 1
END
GO

-- *********************************************
-- ***  Crear Procedimientos Almacenados     ***
-- *********************************************
CREATE PROCEDURE [dbo].[GetPaginasRowNumber]
   @NUM_PAGINA   INT
   ,@TAM_PAGINA   INT
AS
   SET NOCOUNT ON;

   SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED; 

   WITH DRV_TBL AS 
   (
      SELECT 
         ROW_NUMBER() OVER (ORDER BY PAG.FEC_PUBLICACION DESC) AS rownum
         ,PAG.*
      FROM dbo.TBL_PAGINAS AS PAG
   )

   SELECT * FROM DRV_TBL 
   WHERE ROWNUM BETWEEN (@NUM_PAGINA*@TAM_PAGINA)-@TAM_PAGINA+1 AND (@NUM_PAGINA*@TAM_PAGINA)

GO


CREATE PROCEDURE [dbo].[GetCountPaginas]
AS
   SET NOCOUNT ON;

   SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED; 

   DECLARE @CUANTOS INT;

   SELECT @CUANTOS=COUNT(*)
   FROM dbo.TBL_PAGINAS 

   RETURN @CUANTOS;

GO

CREATE PROCEDURE [dbo].[GetPaginasSinPaginar]
AS
   SET NOCOUNT ON;

   SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED; 

   SELECT PAG.*
   FROM dbo.TBL_PAGINAS AS PAG

GO

CREATE PROCEDURE [dbo].[GetPaginasTOPmenorQue]
   @MENOR_QUE         DATETIME2 = NULL
   ,@TAM_PAGINA   INT
AS
   SET NOCOUNT ON;

   SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED; 

   SELECT TOP (@TAM_PAGINA) * 
   FROM dbo.TBL_PAGINAS AS PAG
   WHERE (FEC_PUBLICACION<@MENOR_QUE OR @MENOR_QUE IS NULL)
   ORDER BY FEC_PUBLICACION DESC

GO

CREATE PROCEDURE [dbo].[GetPaginasTOPmayorQue]
   @MAYOR_QUE         DATETIME2 = NULL
   ,@TAM_PAGINA   INT
AS
   SET NOCOUNT ON;

   SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED; 

   SELECT *
   FROM 
   (
	   SELECT TOP (@TAM_PAGINA) * 
	   FROM dbo.TBL_PAGINAS AS PAG
	   WHERE (FEC_PUBLICACION>@MAYOR_QUE OR @MAYOR_QUE IS NULL)
	   ORDER BY FEC_PUBLICACION
   ) AS DRV_TBL
   ORDER BY FEC_PUBLICACION DESC

GO

-- ******************************************************************
-- ***  Ejecuciones de ejemplo de los Procedimientos Almacenados  ***
-- ******************************************************************

-- ***  Sin Paginar
EXEC [GetPaginasSinPaginar]

-- ***  Paginamiento con acceso Aleatorio utilizando ROW_NUMBER
EXEC [GetPaginasRowNumber] @num_pagina=1,@tam_pagina=10
EXEC [GetPaginasRowNumber] @num_pagina=2,@tam_pagina=10

DECLARE @CUANTOS INT
EXEC @CUANTOS=[GetCountPaginas]
SELECT @CUANTOS

-- ***  Paginamiento con acceso Secuencial utilizando TOP, tanto hacia adelante como hacia atrs
EXEC [GetPaginasTOPmenorQue] @tam_pagina=10
EXEC [GetPaginasTOPmenorQue] @menor_que='20170510',@tam_pagina=10
EXEC [GetPaginasTOPmenorQue] @menor_que='20170430',@tam_pagina=10
EXEC [GetPaginasTOPmayorQue] @mayor_que='20170429',@tam_pagina=10
EXEC [GetPaginasTOPmayorQue] @mayor_que='20170509',@tam_pagina=10